from .race_class import RaceClass
from .race_grade import RaceGrade


class RaceLevel:
    def __init__(self, value: RaceGrade | RaceClass):
        if isinstance(value, RaceClass) and value.value == 1:
            raise ValueError("Class 1 race needs a specified grade")

        self.grade = value if isinstance(value, RaceGrade) else RaceGrade(None)
        self.class_ = value if isinstance(value, RaceClass) else RaceClass(1)

    def __str__(self):
        return f"({self.class_.value}) {self.grade if self.grade.value else ''}".strip()

    def __eq__(self, other):
        return self.grade == other.grade and self.class_ == other.class_

    def __gt__(self, other):
        return self.grade > other.grade or self.class_ > other.class_

    def __lt__(self, other):
        return self.grade < other.grade or self.class_ < other.class_



from typing import Self


class RaceClass:
    def __init__(self, value: str | int):
        if not 1 <= int(value) <= 7:
            raise ValueError(f"Class must be between 1 and 7, not {value}")

        self.value = int(value)

    def __str__(self) -> str:
        return f"Class {self.value}"

    def __eq__(self, other: Self) -> bool:
        return self.value == other.value

    def __gt__(self, other: Self) -> bool:
        return self.value < other.value

    def __lt__(self, other: Self) -> bool:
        return self.value > other.value



from .parsing_enum import ParsingEnum


class RacecourseContour(ParsingEnum):
    """
    An enumeration representing the contour of a racecourse.

    """

    UNKNOWN = 0
    FLAT = 1
    DOWNHILL = 2
    UPHILL = 3
    UNDULATING = 4

    # Alternatives
    DOWN = DOWNHILL
    UP = UPHILL



from .aw_going_description import AWGoingDescription
from .dirt_going_description import DirtGoingDescription
from .going_description import GoingDescription
from .turf_going_description import TurfGoingDescription


class Going:
    """
    A class to represent a going.
    """

    Scales = (TurfGoingDescription, AWGoingDescription, DirtGoingDescription)

    def __init__(self, description: str, reading: float | None = None):
        """
        Initialize a Going instance.

        Args:
            description: The description of the going.
            reading: The reading of the going stick.
        """
        self.description = description
        self.reading = reading

        assert (self.primary and self.secondary) or self.primary

    @property
    def primary(self) -> GoingDescription:
        """
        The primary property of the going.

        Returns:
            A value selected from the appropriate going scale.
        """
        key = self._description_parts[0]
        return Going._lookup(key)

    @property
    def secondary(self) -> GoingDescription | None:
        """
        The secondary or 'in places' property of the going.

        Returns:
            A value selected from the appropriate going scale.
        """
        key = self._description_parts[1]
        return Going._lookup(key) if key else None

    @property
    def value(self) -> float:
        """
        A numerical value for the going.

        Returns:
            The value of the going.
        """
        return (
            self.primary.value
            if self.secondary is None
            else (self.primary.value + self.secondary.value) / 2
        )

    @property
    def _description_parts(self) -> list[str]:
        """
        The parts of the description.

        Returns:
            The parts of the description.
        """
        texts = self.description.upper().replace(" IN PLACES", "").split(", ")
        return texts if len(texts) == 2 else texts + [""]

    @classmethod
    def _lookup(cls, key: str) -> GoingDescription:
        """
        Lookup a value in the appropriate going scale.

        Args:
            key: The key to lookup.

        Returns:
            A value selected from the appropriate going scale.

        Raises:
            ValueError: If the key is not found in any of the going scales.
        """
        for scale in Going.Scales:
            try:
                return scale[key]
            except KeyError:
                pass

        raise ValueError(f"Unknown going description: {key}")



from pendulum import DateTime
from .going import Going
from .race_distance import RaceDistance
from .stalls_position import StallsPosition
from .racecourse import Racecourse


class RaceConditions:
    """
    A class for grouping together race conditions into a single object.

    """

    def __init__(
        self,
        *,
        datetime: DateTime,
        racecourse: Racecourse,
        distance: RaceDistance,
        going: Going,
        stalls_position: StallsPosition | None = None,
    ):
        """
        Initialize a RaceConditions instance.

        Args:
            datetime: The datetime of the race
            racecourse: The racecourse on which the race is run
            distance: The race distance
            going: The going of the race
            stalls_position: The position of the stalls on the track

        """
        self.datetime = datetime
        self.racecourse = racecourse
        self.distance = distance
        self.going = going
        self.stalls_position = stalls_position



from .racing_code import RacingCode


class RaceGrade:
    def __init__(
        self, grade: str | int | None, racing_code: RacingCode = RacingCode.FLAT
    ):
        if grade and str(grade).isdigit() and not 1 <= int(grade) < 4:
            raise ValueError(f"Grade must be between 1 and 3, not {grade}")

        if grade and not str(grade).isdigit() and grade != "Listed":
            raise ValueError(f"Grade must be a number or 'Listed', not {grade}")

        self.value = str(grade) if grade else None
        self.racing_code = racing_code

    def __str__(self):
        if not self.value:
            return ""

        title = "Grade" if self.racing_code == RacingCode.NATIONAL_HUNT else "Group"
        return "Listed" if not self.value.isdigit() else f"{title} {self.value}"

    def __eq__(self, other):
        return self.value == other.value

    def __lt__(self, other):
        if not self.value:
            return other.value

        if not self.value.isdigit():
            return other.value.isdigit()

        return other.value.isdigit() and self.value > other.value

    def __gt__(self, other):
        if not self.value:
            return False

        return self.value.isdigit() and (
            not other.value or not other.value.isdigit() or self.value < other.value
        )



from .handedness import Handedness
from .racecourse_contour import RacecourseContour
from .racecourse_shape import RacecourseShape
from .racecourse_style import RacecourseStyle
from .surface import Surface


class Racecourse:
    """
    A class for grouping together racecourse characteristics into a single object.

    """

    def __init__(self, name: str, surface: Surface | str, **kwargs):
        """
        Initialize a Racecourse instance.

        Args:
            name: The name of the racecourse
            surface: The surface on which the racecourse is run
            handedness: The handedness of the racecourse
            contour: The contour of the racecourse
            shape: The shape of the racecourse
            style: The style of the racecourse

        """
        handedness = kwargs.get("handedness", "unknown")
        contour = kwargs.get("contour", "unknown")
        shape = kwargs.get("shape", "unknown")
        style = kwargs.get("style", "unknown")

        self.name = name
        self.surface = Surface[surface] if isinstance(surface, str) else surface
        self.handedness = (
            Handedness[handedness] if isinstance(handedness, str) else handedness
        )
        self.contour = (
            RacecourseContour[contour] if isinstance(contour, str) else contour
        )
        self.shape = RacecourseShape[shape] if isinstance(shape, str) else shape
        self.style = RacecourseStyle[style] if isinstance(style, str) else style



class Draw:
    def __init__(self, value: int | str) -> None:
        """
        Initialize a Draw instance.

        Args:
            value: The stall number in which the horse is drawn.
        """
        if not str(int(value)) == str(value):
            raise ValueError("Draw must be an integer value")

        self.value = int(value)

    def __gt__(self, other: "Draw") -> bool:
        """
        Returns True if the draw is higher than the other draw.

        Args:
            other: The other draw to compare against.
        """
        return self.value > other.value

    def __lt__(self, other: "Draw") -> bool:
        """
        Returns True if the draw is lower than the other draw.

        Args:
            other: The other draw to compare against.
        """
        return self.value < other.value
